@@ -10,10 +10,12 @@ extern void exit( int );
 extern unsigned int strlen( const char * );
 extern void prI( int n, int leading );
 #define prIln(n) {prI(n,0);write(1,"\n",1);}
+#if 0
 #define assert(x) if(x==0){write(1,"\nASSERTION FAILED! file ",24);\
                            write(1,__FILE__,strlen(__FILE__)); write(1," line ",6);\
                            prIln(__LINE__);\
                            exit(1);}
+#endif
 
 extern pr int __mp_dummyvar;        /* make multiprefix work correctly */
 #define mpadd(a,b) (__mp_dummyvar = mpadd((a),b))
@@ -154,27 +156,45 @@ typedef struct {             /* reader-writer-locks */
    fair_lock writerlock;         /* current and next number */
 } rw_lock;
 
+typedef struct {             /* reader-writer-locks */
+   unsigned int readercounter;   /* Bits 0-29 used as reader counter */
+                                 /* Bit 30 is used as writer flag */
+   fair_lock writerlock;         /* current and next number */
+   int deleteflag;               /* if 1, this lock is going to be deleted */
+} rwd_lock;
+
 #define RW_READ 0
 #define RW_WRITE 1
+#define RW_DELETE 2
 
 /* the following routines are implemented in forklib2.asm */
 
-#define simple_lock_init(slock) slock=0
+extern simple_lock *new_simple_lock( void );
+#define simple_lock_init(pslock) *(pslock)=0
 extern void simple_lockup( simple_lock * );
-#define simple_unlock(slock) *(slock)=0
+#define simple_unlock(pslock) *(pslock)=0
 
+extern fair_lock *new_fair_lock( void );
 extern void fair_lock_init(fair_lock *lock);
 extern void fair_lockup( fair_lock * );
-extern int fair_unlock( fair_lock * );
+extern int fair_lockup_delflag( fair_lock *, int * );
+extern void fair_unlock( fair_lock * );
 
+extern safe_lock *new_safe_lock( void );
 #define safe_lock_init(slock) slock={0,-1}
 extern void safe_lockup( safe_lock * );
-extern int safe_unlock( safe_lock * );
+extern int safe_unlock( safe_lock * );   /* returns an error code */
 
 /* the following routines are implemented in async.c: */
+extern rw_lock *new_rw_lock( void );
 extern void rw_lock_init( rw_lock * );
 extern void rw_lockup( rw_lock *, int );
-extern int rw_unlock( rw_lock *, int, int );
+extern void rw_unlock( rw_lock *, int, int );
+
+extern rwd_lock *new_rwd_lock( void );
+extern void rwd_lock_init( rwd_lock * );
+extern int rwd_lockup( rwd_lock *, int );  /* returns 0 if failed, 1 if lock acquired */
+extern void rwd_unlock( rwd_lock *, int, int );
 
 
 /* critical sections: some useful macros */
@@ -189,9 +209,6 @@ extern int rw_unlock( rw_lock *, int, int );
 /*or*/
 #define _enter_crit_sect(adr) safe_lockup(adr)
 #define _exit_crit_sect(adr) safe_unlock(adr)
-/*or*/
-#define _enter_crit_sect(adr) rw_lockup(adr,RW_WRITE)
-#define _exit_crit_sect(adr) rw_unlock(adr,RW_WRITE,10)
 #endif
 
 /*#define DEBUG_CRIT_SECT      /* print locking and unlocking events on screen */
@@ -203,13 +220,7 @@ extern int rw_unlock( rw_lock *, int, int );
 #define enter_crit_sect(adr) _enter_crit_sect(adr);
 #endif
 
-#ifdef DEBUG_CRIT_SECT
-#define exit_crit_sect(adr) {_exit_crit_sect(adr);write(1,"Ausgang",7);prIln(__LINE__);} 
-#else
-#define exit_crit_sect(adr) assert(!_exit_crit_sect(adr)) /*alock=0*/
-#endif
-
-#undef assert
+/*#undef assert*/
 #undef prIln
 
 
