@@ -23,14 +23,42 @@
  */
 
 #include <fork.h>
+#include <io.h>
 
 pr int __mp_dummyvar;   /* bugfix to make multipraefix work properly */
 
+simple_lock *new_simple_lock( void )
+{
+  simple_lock *ret = (simple_lock *)shmalloc( sizeof( simple_lock ));
+  *ret = 0;
+  return ret;
+}
+
+fair_lock *new_fair_lock( void )
+{
+  fair_lock *ret = (fair_lock *)shmalloc( sizeof( fair_lock ));
+  ret->nextnum = ret->actnum = 0;
+  return ret;
+}
+
+
 void fair_lock_init( fair_lock *pfl )
 {
   pfl->nextnum = pfl->actnum = 0;
 }
 
+
+int fair_lockup_delflag( fair_lock *plock, int *delflag )
+{
+  // same as fair_lockup( plock ) (see forklib2.asm)
+  // but aborts and returns 0 when delflag becomes 1 while waiting
+  unsigned int myticket = mpadd( &(plock->nextnum), 1 );
+  while (plock->actnum < myticket)     /* wait and check */
+     if ( *delflag != 0)  return 0;
+  return 1;                            /* OK, obtained the lock */
+}
+
+
 /* ############### definition of reader/writer-lock  ############### */
 
 /* we use "mpadd" and "syncor" to acces the memory cell holding the number
@@ -57,6 +85,15 @@ void rw_lock_init( rw_lock *lock)
    lock->writerlock.actnum = 0;    /* init fair lock */
 }
 
+rw_lock *new_rw_lock( void )
+{
+  rw_lock *ret = (rw_lock *)shmalloc( sizeof( rw_lock ));
+  ret->readercounter = 0;
+  ret->writerlock.nextnum = ret->writerlock.actnum = 0;
+  return ret;
+}
+
+
 
 void rw_lockup( rw_lock *lock, int ptype)
 {
@@ -99,7 +136,7 @@ void rw_lockup( rw_lock *lock, int ptype)
 } /* _rw_lockup() */
 
 
-int rw_unlock( rw_lock *lock, int ptype, int wait)
+void rw_unlock( rw_lock *lock, int ptype, int wait)
 {
  int dummy;
  if(ptype == RW_READ)    /* reader-unlock */
@@ -116,6 +153,88 @@ int rw_unlock( rw_lock *lock, int ptype, int wait)
 } /* _rw_unlock() */
 
 
+
+// RWD - Lock: a generalization of the RW-Lock  C. Kessler 9903
+
+void rwd_lock_init( rwd_lock *lock)
+{
+   lock->readercounter = 0;
+   lock->writerlock.nextnum = 0;   /* init fair lock */
+   lock->writerlock.actnum = 0;    /* init fair lock */
+   lock->deleteflag = 0;
+}
+
+rwd_lock *new_rwd_lock( void )
+{
+  rwd_lock *ret = (rwd_lock *)shmalloc( sizeof( rwd_lock ));
+  ret->readercounter = 0;
+  ret->writerlock.nextnum = ret->writerlock.actnum = 0;
+  ret->deleteflag = 0;
+  return ret;
+}
+
+
+
+int rwd_lockup( rwd_lock *lock, int ptype)
+{
+ int wait;
+ pprintf("call rwd_lockup( %p, %s )\n", lock, (ptype==RW_READ)?"RW_READ":
+          (ptype==RW_WRITE)?"RW_WRITE":"RW_DELETE");
+ if(ptype == RW_READ) {         /* reader-lock */
+    wait = 1; /* wait = TRUE */
+    while(wait) {
+       wait = 0; /* wait = FALSE */
+       /* wait for writer to leave the lock */
+       while( mpadd(&(lock->readercounter), 0) & __RW_FLAG__ )
+          if (lock->deleteflag) return 0;  /* fail */
+       /* writer has finished - now try to catch lock */
+       if( mpadd(&(lock->readercounter), 1) & __RW_FLAG__ ) {
+          /* another writer was faster than me */
+          mpadd( &(lock->readercounter), -1); 
+          /* undo lock */
+          if (lock->deleteflag) return 0;  /* fail */
+          wait = 1; /* wait = TRUE */
+       }
+    }
+    pprintf("rwd_lockup( %p, RW_READ ) succeeded\n", lock );
+ }
+ else {      /* writer-lock */
+    /* wait for writer to leave the lock */
+    if (!fair_lockup_delflag(&(lock->writerlock), &(lock->deleteflag)))
+        return 0;   /* fail */
+    /*  set flag to indicate to the readers that I want to have the lock */
+    syncor( (int *)&(lock->readercounter), __RW_FLAG__ );
+    /* wait for readers to leave the lock */
+    while( mpadd( &(lock->readercounter), 0) & (__RW_FLAG__-1))
+       if (lock->deleteflag) return 0;  /* fail */
+    if (ptype == RW_DELETE) {
+       lock->deleteflag = 1;   /*stg at mo=1, all other accesses ldg at mo=0*/
+    }
+    pprintf("rwd_lockup( %p, %s ) succeeded\n", lock, (ptype==RW_WRITE)?"RW_WRITE":"RW_DELETE");
+ }
+ return 1;
+} /* rwd_lockup() */
+
+
+void rwd_unlock( rwd_lock *lock, int ptype, int wait)
+{
+ pprintf("rwd_unlock( %p, %s )\n", lock, (ptype==RW_READ)?"RW_READ":
+          (ptype==RW_WRITE)?"RW_WRITE":"RW_DELETE");
+ if(ptype == RW_READ)    /* reader-unlock */
+    mpadd ( &(lock->readercounter), -1);
+ else {   /* writer-unlock */
+    int i;
+    /* free lock for readers */
+    mpadd ( &(lock->readercounter), -__RW_FLAG__);      
+    /* wait loop */
+    for(i = 0; i < wait; i++);
+    /* free for writers */
+    fair_unlock(&(lock->writerlock));
+ }
+} /* rwd_unlock() */
+
+
+
 #if 0
 
 /* ############### parallel loop ############### */
