@@ -708,6 +708,15 @@ int fclose ( FILE *fp )
  return errcode;
 }
  
+int fflush( FILE *fp )
+{
+ assert( fp );
+ if (fp->buffer) {
+    pprintf("fflush: buffered output not supported, sorry \n");   
+    return 1;
+ }
+ return 0;
+}
 
 int fputc ( int c, FILE *fp )
 {
