@@ -631,15 +631,18 @@ Node listnodes(tp, tlab, flab) Tree tp; {
    case EXLGRP: case SIZE: case IBP: case FLUSHFRAMES:
    case BEGSEQ: case ENDSEQ:
    case SHSPACE: /*970902*/
+   case JBLOCKa: case JBLOCKc: case JBLOCKd:
+   case JBLOCKe: case JBLOCKf: case JBLOCKg: case JBLOCKh:
       p = newnode(tp->op, 0, 0, 0);
       p->synchron = tp->synchron;      /*durchreichen CWK*/
       p->resetshoffset = 0; /*970903*/
       if (op==ENGRP || op==SPROC /* nicht auskommentiert 970908*/
+          || op==JBLOCKf   /*990126*/
           || op==ENLGRP || op==NEXTCASE) {
           p->resetshoffset = 1;        /*970903*/
       } 
       break;
-   case MKGRP: case SPROC: case ILOG2:
+   case MKGRP: case SPROC: case ILOG2: case JBLOCKb:
       assert(tlab == 0 && flab == 0);
       l = listnodes(tp->kids[0], 0, 0);
       p = node(tp->op, l, 0, 0);
