@@ -1,7 +1,7 @@
 /* C compiler: main program */
 
-#include "version.h"
 #include "c.h"
+#include "version.h"
 
 extern int STARTED_PROCS;    /*CWK*/
 
@@ -9,6 +9,7 @@ int Aflag;         /* >= 0 if -A specified */
 int Pflag;         /* != 0 if -P specified */
 int Kflag;         /* != 0 if -K specified CWK ->generate consistency checks*/
 int mflag;         /* != 0 if -m specified CWK ->sets care_about_modulo */
+int Tflag=0;       /* != 0 if -T specified 990119 -> gen code for access stat*/
 int isFORKmodul = 0;   /* != 0 if -FORK specified CWK */
 int glevel;         /* == [0-9] if -g[0-9] specified */
 int xref;         /* != 0 for cross-reference data */
@@ -144,8 +145,10 @@ static int doargs(argc, argv) char *argv[]; {
          }
       } else if (strncmp(argv[i], "-t", 2) == 0)
          traceinit(&argv[i][2]);
+      else if (strncmp(argv[i], "-T", 2) == 0)
+         Tflag = 1;
       else if (strcmp(argv[i], "-v") == 0)
-         fprint(2, "%s version %d.%d\n", progname, VERSION>>8, VERSION&0xff);
+         fprint(2, "%s version %s\n", progname, VERSION );
       else if (strncmp(argv[i], "-s", 2) == 0)
          density = strtod(&argv[i][2], (char **)0);
       else if (strncmp(argv[i], "-e", 2) == 0) {
